#ifndef _ROADCONNECTOR_H_
#define _ROADCONNECTOR_H_

#include <QCAR/Vectors.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include "RoadPiece.h"

class RoadConnector{
public:
	static const int CONNECTION_POINTS = 25;
	static Texture* roadTexture;
	static Texture* sideTexture;
	static float EXIT_SPEED;
	static float LIFT_HEIGHT;
	static float LIFT_WIDTH;
private:
	// geometry
	float connectorPoints[CONNECTION_POINTS*3];
	float connectorUpperBounds[2][CONNECTION_POINTS*3];
	float connectorLowerBounds[2][CONNECTION_POINTS*3];

	RoadPiece* srcRP;
	RoadPiece* dstRP;
	
	int srcPtIdx;
	int dstPtIdx;
	float srcPt[3];
	float dstPt[3];
	float srcPtDir[3];
	float dstPtDir[3];

	QCAR::Matrix44F oldOffsetMatrix;

	void findBestConnectionPoints();

	void calculateHermitePoints(GLfloat* control, GLfloat* result, int steps);
	void calculateRoadGeometry(const QCAR::Vec3F& srcDir, const QCAR::Vec3F& dstDir, QCAR::Matrix44F& offsetMat);

	QCAR::Matrix44F lowPassMatrix(const QCAR::Matrix44F& newMatrix, const QCAR::Matrix44F& oldMatrix);

	//spline hasn't been calculated yet
	bool isValid;
public: 
	bool isShowing();
	float* getConnectorPoints();
	QCAR::Matrix44F& getMVMatrix();
	void generateRoadConnection(bool lift=false, int* points=NULL);
	void liftRoad();
	RoadConnector(RoadPiece* src, RoadPiece* dst);
	void draw(QCAR::Matrix44F& projectionMatrix, const RoadGLParams& params, bool temp=false, bool deleting=false);
	bool isConnectedTo(int idx1, int idx2);
	~RoadConnector();
};

#endif
