#ifndef _ROADMANAGER_H_
#define _ROADMANAGER_H_

#include <QCAR/QCAR.h>
#include <QCAR/Renderer.h>
#include <QCAR/Trackable.h>
#include <QCAR/TrackableResult.h>
#include <QCAR/MarkerResult.h>
#include <QCAR/Tool.h>
#include <QCAR/MarkerTracker.h>
#include <QCAR/TrackerManager.h>
#include <QCAR/Marker.h>
#include <QCAR/VideoBackgroundConfig.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include "../Texture.h"
#include "RoadPiece.h"
#include "RoadConnector.h"
#include "Structs.h"

#include <vector>

// states for the selection state machine
#define STATE_NONE_SELECTED 0
#define STATE_ONE_SELECTED 1
#define STATE_TWO_SELECTED 2

class RoadManager{
private:
	// gl shader handles struct
	RoadGLParams	glParams;

	// textures
	int				textureCount;
	Texture**		textures;
	
	// road pieces, this vector is static in each run
	// road pieces are allocated on init
	std::vector<RoadPiece*>	roadPieces;

	// road connectors (splines), this is dynamic
	std::vector<RoadConnector*>	roadConnectors;

	// this is a pointer to a temporary road connector,
	// which is drawn before adding a new one or deleting an existing one
	RoadConnector* tempRoadConnector;
	// is this temporary for deletion or creation
	bool isTempForDeletion;

	// selected road pieces
	int selectedRP[2];
	// state of the selection for the state machine
	int selectionState;

	void projectScreenPointToPlane(QCAR::Vec2F point, QCAR::Vec3F planeCenter, QCAR::Vec3F planeNormal,
									QCAR::Vec4F &lineStart, QCAR::Vec4F &lineEnd, QCAR::Matrix44F& inverseProjMatrix,
									float screenWidth, float screenHeight);
	void handleSelectedRP(int idx);
public:
	RoadManager();
	void initRoadPieces(QCAR::MarkerTracker* markerTracker);
	void initRendering();
	void initTextures(int size);
	void drawRoads(QCAR::State state, QCAR::Matrix44F& projectionMatrix);
	void addTexture(int index,Texture* _texture);
	void deleteTextures();
	void handleTouchEvent(TouchEvent* touch, QCAR::Matrix44F& inverseProjectionMatrix, float screenWidth, float screenHeight);
	~RoadManager();
};


#endif