#ifndef _ROADPIECE_H_
#define _ROADPIECE_H_

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include <QCAR/Vectors.h>
#include <QCAR/Matrices.h>
#include <QCAR/MarkerTracker.h>

#include "../Texture.h"
#include "Structs.h"

class RoadPiece
{
protected:
	static const int MAX_CON_PTS = 4;
	// this should be unique!
	int				pieceIdx;
	bool			isShowing;
	bool			isSelected;
	bool			busyConPts[MAX_CON_PTS];
	QCAR::Matrix44F	modelViewMatrix;
	Texture*		texture;

	// this is to be used by draw function
	virtual void setGLArrays(RoadPieceGLArrays* arrays) = 0;
	virtual const float* getBBVertices() = 0;

	bool checkIntersectionLine(QCAR::Vec3F& pointA, QCAR::Vec3F& pointB);
	bool isSeparatingAxisLine(QCAR::Vec3F axis, QCAR::Vec3F pointA, QCAR::Vec3F pointB);
public:
	RoadPiece(QCAR::MarkerTracker* markerTracker, Texture* pieceTexture, int idx);
	int getIdx();
	void setShowing(bool showing);
	bool isRoadShowing();
	bool isRoadSelected();
	bool isConPtBusy(int idx);
	void toggleBusyConPts(int idx);
	bool hasAvailableConPts();
	void setSelected(bool selected);
	void toggleSelected();
	void updateModelView(const QCAR::Matrix44F& modelView);
	QCAR::Matrix44F& getModelView();
	void draw(QCAR::Matrix44F& projectionMatrix,const RoadGLParams& params);
	bool checkForIntersection(QCAR::Vec4F& lineStart, QCAR::Vec4F& lineEnd);

	// pure virtual functions
	virtual int				getConPtsCount() = 0;
	virtual const float*	getConPts() = 0;
	virtual const float*	getConPtsDirs() = 0;
};

#endif //_ROADPIECE_H_